
# q-expansion of newform 3784.1.em.b, downloaded from the LMFDB on 22 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as list of pairs
# Each pair (c, e) corresponds to c*zeta^e

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import CyclotomicField
    K = CyclotomicField(poly_data, "z")
    convert_elt_to_field = lambda elt: sum(c * K.gens()[0]**e for c,e in elt)
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 3784
weight = 1
poly_data = 210
hecke_ring_character_values = [[2839, [[-1, 0]]], [1893, [[-1, 0]]], [1377, [[-1, 63]]], [89, [[1, 40]]]]
aps_data = [[[-1, 99]], [[1, 8], [-1, 11]], [], [], [[1, 62]], [], [[-1, 23], [-1, 69]], [[-1, 49], [1, 60]], [], [], [], [], [[1, 26], [1, 88]], [[-1, 1]], [], [], [[1, 40], [-1, 101]], [], [[1, 58], [1, 72]], [], [[-1, 59], [-1, 87]], [], [[-1, 35], [-1, 47]], [[-1, 9], [-1, 31]], [[-1, 53], [-1, 55]], [], [], [[-1, 73], [1, 86]], [], [[1, 100], [1, 104]], [], [[-1, 53], [1, 82]], [[1, 30], [1, 102]], [[1, 28], [1, 78]], [], [], [], [[1, 2], [1, 36]], [], [], [[-1, 29], [1, 90]], [], [], [[-1, 21], [1, 96]], [], [], [[-1, 33], [1, 90]], [], [[-1, 21], [1, 100]], [], [[1, 18], [1, 50]], [], [[1, 52], [-1, 103]], [[1, 49]], [[-1, 7], [1, 14]], [], [], [], [], [[1, 12], [1, 40]], [[1, 92], [1, 102]], [], [[1, 16], [1, 54]], [], [[1, 68], [1, 74]], [], [[-1, 51], [1, 74]], [[1, 20], [1, 36]], [[-1, 39], [-1, 43]], [], [[-1, 23], [-1, 57]], [], [], [], [[1, 2], [-1, 25]], [], [], [], [[-1, 37], [1, 70]], [[-1, 59], [-1, 79]], [[-1, 9], [1, 36]], [], [], [[-1, 65], [-1, 101]], [], [[-1, 13], [-1, 91]], [[-1, 1], [-1, 87]], [[-1, 77], [-1, 85]], [], [], [[1, 46], [1, 66]], [], [], [[1, 32], [-1, 99]], [[-1, 87], [-1, 89]], [], [], [[1, 0], [1, 76]], [[1, 18], [-1, 59]], [], [[-1, 5], [1, 74]], [], [[1, 48], [-1, 75]], [[-1, 75], [1, 94]], [[-1, 35], [-1, 65]], [[-1, 49], [-1, 103]], [[-1, 45], [-1, 71]], [[-1, 3], [-1, 77]], [], [[1, 22], [1, 104]], [], [], [[1, 28], [-1, 67]], [[-1, 55], [-1, 79]], [], [[-1, 1], [1, 38]], [[1, 10], [1, 62]], [], [], [[-1, 73], [-1, 97]], [], [[1, 26], [-1, 27]], [], [[-1, 5], [1, 50]], [[-1, 27], [1, 76]], [], [], [], [], [], [[1, 6], [1, 42]], [], [], [], [[-1, 7], [1, 76]], [[-1, 25], [1, 30]], [], [[1, 64], [-1, 103]], [], [[-1, 33], [-1, 99]], [[-1, 21], [1, 28]], [], [], [[-1, 21], [1, 22]], [], [], [], [[-1, 11], [-1, 29]], [[1, 38], [-1, 67]], [], [], [[-1, 11], [1, 64]], [[1, 38], [1, 56]], [], [[-1, 19], [-1, 83]], [], [], [[1, 18], [1, 74]], [[1, 62], [-1, 81]], [], [[-1, 37], [1, 68]], [[-1, 67], [1, 94]], [], [[-1, 37], [1, 52]], [[1, 24], [1, 34]], [], [], [], [[1, 28], [1, 66]], [], [[-1, 69], [-1, 77]], [], [], [[1, 12], [-1, 93]], [], [[1, 8], [-1, 63]], [[-1, 75], [-1, 97]], [], [], [], [], [[-1, 29], [-1, 49]], [], [[1, 12], [1, 42]], [], [], [], [[-1, 61], [-1, 99]], [[1, 80], [-1, 91]], [], [[1, 72], [-1, 81]], [[1, 4], [-1, 5]], [[1, 38], [-1, 89]], [], [[-1, 15], [-1, 35]], [[-1, 31], [1, 56]], [[-1, 43], [1, 60]], [], [[1, 73]], [], [], [], [], [[-1, 27], [-1, 45]], [[-1, 57], [-1, 85]], [], [], [[1, 60], [-1, 101]], [[-1, 9], [1, 54]], [[-1, 39], [-1, 87]], [[1, 68], [1, 72]], [], [], [[1, 20], [-1, 93]], [], [[-1, 1], [-1, 99]], [], [[-1, 3], [-1, 71]], [], [], [], [], [[1, 20], [-1, 65]], [], [[1, 82], [1, 92]], [], [[1, 10], [1, 34]], [], [], [[1, 46], [1, 104]], [], [[-1, 19], [1, 42]], [], [[-1, 63], [1, 88]], [[-1, 35], [-1, 103]], [], [[1, 42], [-1, 99]], [], [[-1, 51], [-1, 103]], [], [[-1, 73], [-1, 79]], [[-1, 33], [1, 80]], [], [], [[1, 26], [1, 92]], [], [], [[1, 30], [-1, 43]], [[-1, 41], [-1, 101]], [], [], [[1, 6], [1, 16]], [], [[-1, 5], [1, 84]], [], [[1, 62], [1, 96]], [], [[-1, 53], [-1, 87]], [], [[-1, 65], [-1, 71]], [], [[1, 11]], [], [], [[-1, 27], [-1, 67]], [[1, 40], [-1, 77]], [], [[-1, 69], [1, 78]], [[-1, 75], [-1, 91]], [], [], [[1, 88], [-1, 95]], [[1, 16], [1, 90]], [], [[-1, 53], [1, 54]], [], [[-1, 19], [1, 48]], [], [[-1, 49], [1, 90]], [[-1, 83], [1, 98]], [], [], [], [], [[1, 34], [-1, 55]], [], [[1, 54], [-1, 55]], [], [], [[1, 2], [-1, 17]], [], [[1, 4], [-1, 7]], [[1, 34], [-1, 41]], [[1, 8], [1, 94]], [], [], [], [], [[1, 44], [-1, 61]], [[1, 12], [-1, 89]], [[-1, 3], [1, 50]], [], []]
